<?php

namespace InSegment\WorkersFramework\Drivers\RabbitMq;

use InSegment\WorkersFramework\Interfaces\ConsumerInterface;
use InSegment\WorkersFramework\Interfaces\QueueDriverInterface;
use InSegment\WorkersFramework\Interfaces\DeclareQueueBuilderInterface;
use Shavshukov\EventAppBase\Service\ErrorReport;

use Bunny\Protocol\MethodBasicConsumeOkFrame;
use Bunny\Async\Client;
use Bunny\Channel;
use Bunny\Message;
use React\Promise;

class AsyncChannel extends ChannelDriver implements QueueDriverInterface
{
    /**
     * Create instance of RabbitMq service driver connected by async channel
     * 
     * @param \InSegment\WorkersFramework\Drivers\RabbitMq\Connector $connector
     * @param \React\Promise\PromiseInterface<\Bunny\Channel> $channelPromise
     * @return \React\Promise\PromiseInterface<\InSegment\WorkersFramework\Drivers\RabbitMq\AsyncChannel>
     */
    public static function create(Connector $connector, Promise\PromiseInterface $channelPromise): Promise\PromiseInterface
    {
        return $channelPromise->then(function (Channel $channel) use ($connector) {
            return $channel->qos(
                $connector->getPrefetchSize(),
                $connector->getPrefetchCount()    
            )->then(function () use ($channel) {
                return new static($channel);
            });
        });
    }

    /**
     * {@inheritDoc}
     */
    public function declare(): DeclareQueueBuilderInterface
    {
        return new DeclareBuilder($this->channel);
    }
    
    /**
     * Subscribe to queue and return consumer tag
     * 
     * @param string $queueName
     * @param \InSegment\WorkersFramework\Interfaces\ConsumerInterface $receiver
     * @return \React\Promise\PromiseInterface<string>
     */
    public function subscribe(string $queueName, ConsumerInterface $receiver)
    {
        return $this->channel->consume(
            function (Message $message, Channel $channel, Client $client) use ($receiver, $queueName) {
                (new Promise\Promise(function ($resolver) use ($message, $channel, $queueName, $receiver) {
                    $messageWrapped = new QueueMessage($message, $channel, $queueName);
                    Promise\resolve($receiver->receiveMessage($messageWrapped, $this))->then($resolver);
                }))->then(null, function (\Throwable $e) use ($channel, $message) {
                    ErrorReport::report($e, false);
                    $channel->reject($message, false);
                });
            },
            $queueName
        )->then(function (MethodBasicConsumeOkFrame $response) {
            return $response->consumerTag;
        });
    }


}
